using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinTree;
using Infragistics.Win;

namespace Trane.BrusselsHQ.WinForms.Components.TreeView
{
    /// <summary>
    /// This class is mainly an UltraTree with drag and drop capabilities.
    /// Implements: highlighting of the node that will be dropped on or
    /// drawing a line where the node will be placed.
    /// </summary>
    /// <remarks>
    /// This relies on the class <see cref="MyDrawFilter"/>, which implements IUIElementDrawFilter.
    /// </remarks>
    public class TFwkTree : UltraTree
    {
        /// <summary>
        /// MyDrawFilter is used to draw the selected node or the line where the node that is
        /// being dragged will fall. This gives a good feedback for the user.
        /// </summary>
        private class MyDrawFilter : Infragistics.Win.IUIElementDrawFilter
        {
            //This class has to implement the DrawFilter Interface
            //so it can be used as a DrawFilter by the tree

            #region Event declarations
            public delegate void QueryStateAllowedForNodeEventHandler(object sender, QueryStateAllowedForNodeEventArgs e);

            public event System.EventHandler Invalidate;
            public event QueryStateAllowedForNodeEventHandler QueryStateAllowedForNode;

            //Used by the QueryStateAllowedForNode event
            public class QueryStateAllowedForNodeEventArgs : System.EventArgs
            {
                public UltraTreeNode Node;
                public DropLinePositionEnum DropLinePosition;
                public DropLinePositionEnum StatesAllowed;
            }

            #endregion

            #region Construction / Cleanup
            public MyDrawFilter()
            {
                //Initialize the properties to the defaults
                InitProperties();
            }

            //Initialize the properties to the defaults
            private void InitProperties()
            {
                mvarDropHighLightNode = null;
                mvarDropLinePosition = DropLinePositionEnum.None;
                mvarDropHighLightBackColor = System.Drawing.SystemColors.Highlight;
                mvarDropHighLightForeColor = System.Drawing.SystemColors.HighlightText;
                mvarDropLineColor = System.Drawing.SystemColors.ControlText;
                mvarEdgeSensitivity = 0;
                mvarDropLineWidth = 2;
            }


            //Clean up
            public void Dispose()
            {
                mvarDropHighLightNode = null;
            }

            #endregion

            //The DropHighLightNode is a reference to the node the
            //Mouse is currently over
            private UltraTreeNode mvarDropHighLightNode;
            public UltraTreeNode DropHightLightNode
            {
                get
                {
                    return mvarDropHighLightNode;
                }
                set
                {
                    //If the Node is being set to the same value,
                    // just exit
                    if (mvarDropHighLightNode.Equals(value))
                    {
                        return;
                    }
                    mvarDropHighLightNode = value;
                    //The DropNode has changed.
                    PositionChanged();
                }
            }

            private DropLinePositionEnum mvarDropLinePosition;
            public DropLinePositionEnum DropLinePosition
            {
                get
                {
                    return mvarDropLinePosition;
                }
                set
                {
                    //If the position is the same as it was, 
                    //just exit
                    if (mvarDropLinePosition == value)
                    {
                        return;
                    }
                    mvarDropLinePosition = value;
                    //The Drop Position has changed
                    PositionChanged();
                }
            }

            #region Simple Properties

            //The width of the DropLine
            private int mvarDropLineWidth;
            public int DropLineWidth
            {
                get
                {
                    return mvarDropLineWidth;
                }
                set
                {
                    mvarDropLineWidth = value;
                }
            }

            //The BackColor of the DropHighLight node
            //This only affect the node when it is being dropped On. 
            //Not Above or Below. 
            private System.Drawing.Color mvarDropHighLightBackColor;
            public System.Drawing.Color DropHighLightBackColor
            {
                get
                {
                    return mvarDropHighLightBackColor;
                }
                set
                {
                    mvarDropHighLightBackColor = value;
                }
            }

            //The ForeColor of the DropHighLight node
            //This only affect the node when it is being dropped On. 
            //Not Above or Below. 
            private System.Drawing.Color mvarDropHighLightForeColor;
            public System.Drawing.Color DropHighLightForeColor
            {
                get
                {
                    return mvarDropHighLightForeColor;
                }
                set
                {
                    mvarDropHighLightForeColor = value;
                }
            }

            //The color of the DropLine
            private System.Drawing.Color mvarDropLineColor;
            public System.Drawing.Color DropLineColor
            {
                get
                {
                    return mvarDropLineColor;
                }
                set
                {
                    mvarDropLineColor = value;
                }
            }

            //Determines how close to the top or bottom edge of a node
            //the mouse must be to be consider dropping Above or Below
            //respectively. 
            //By default the top 1/3 of the node is Above, the bottom 1/3
            //is Below, and the middle is On. 
            private int mvarEdgeSensitivity;
            public int EdgeSensitivity
            {
                get
                {
                    return mvarEdgeSensitivity;
                }
                set
                {
                    mvarEdgeSensitivity = value;
                }
            }

            #endregion // Simple properties


            //When the DropNode or DropPosition change, we fire the
            //Invalidate event to let the program know to invalidate
            //the Tree control. 
            //This is neccessary since the DrawFilter does not have a 
            //reference to the Tree Control (although it probably could)
            private void PositionChanged()
            {
                // if nobody is listening then just return
                //
                if (null == this.Invalidate)
                    return;

                System.EventArgs e = System.EventArgs.Empty;

                this.Invalidate(this, e);
            }

            //Set the DropNode to Nothing and the position to None. This
            //Will clear whatever Drophighlight is in the tree
            public void ClearDropHighlight()
            {
                SetDropHighlightNode(null, DropLinePositionEnum.None);
            }

            //Call this proc every time the DragOver event of the 
            //Tree fires. 
            //Note that the point passed in MUST be in Tree coords, not
            //form coords
            public void SetDropHighlightNode(UltraTreeNode Node, System.Drawing.Point PointInTreeCoords)
            {
                //The distance from the edge of the node used to 
                //determine whether to drop Above, Below, or On a node
                int DistanceFromEdge;

                //The new DropLinePosition
                DropLinePositionEnum NewDropLinePosition;

                DistanceFromEdge = mvarEdgeSensitivity;
                //Check to see if DistanceFromEdge is 0
                if (DistanceFromEdge == 0)
                {
                    //It is, so we use the default value - one third. 
                    DistanceFromEdge = Node.Bounds.Height / 3;
                }

                //Determine which part of the node the point is in
                if (PointInTreeCoords.Y < (Node.Bounds.Top + DistanceFromEdge))
                {
                    //Point is in the top of the node
                    NewDropLinePosition = DropLinePositionEnum.AboveNode;
                }
                else
                {
                    if (PointInTreeCoords.Y > ((Node.Bounds.Bottom - DistanceFromEdge) - 1))
                    {
                        //Point is in the bottom of the node
                        NewDropLinePosition = DropLinePositionEnum.BelowNode;
                    }
                    else
                    {
                        //Point is in the middle of the node
                        NewDropLinePosition = DropLinePositionEnum.OnNode;
                    }
                }

                //Now that we have the new DropLinePosition, call the
                //real proc to get things rolling
                SetDropHighlightNode(Node, NewDropLinePosition);
            }

            private void SetDropHighlightNode(UltraTreeNode Node, DropLinePositionEnum DropLinePosition)
            {
                //Use to store whether there have been any changes in 
                //DropNode or DropLinePosition
                bool IsPositionChanged = false;

                try
                {
                    //Check to see if the nodes are equal and if 
                    //the dropline position are equal
                    if (mvarDropHighLightNode != null && mvarDropHighLightNode.Equals(Node) && (mvarDropLinePosition == DropLinePosition))
                    {
                        //They are both equal. Nothing has changed. 
                        IsPositionChanged = false;
                    }
                    else
                    {
                        //One or both have changed
                        IsPositionChanged = true;
                    }
                }
                catch
                {
                    //If we reach this code, it means mvarDropHighLightNode 
                    //is null, so it could not be compared
                    if (mvarDropHighLightNode == null)
                    {
                        //Check to see if Node is nothing, so we//ll know
                        //if Node = mvarDropHighLightNode
                        IsPositionChanged = !(Node == null);
                    }
                }

                //Set both properties without calling PositionChanged
                mvarDropHighLightNode = Node;
                mvarDropLinePosition = DropLinePosition;

                //Check to see if the PositionChanged
                if (IsPositionChanged)
                {
                    //Position did change.
                    PositionChanged();
                }
            }

            #region IUIElementDrawFilter implementation

            //Only need to trap for 2 phases:
            //AfterDrawElement: for drawing the DropLine
            //BeforeDrawElement: for drawing the DropHighlight
            Infragistics.Win.DrawPhase Infragistics.Win.IUIElementDrawFilter.GetPhasesToFilter(ref Infragistics.Win.UIElementDrawParams drawParams)
            {
                return Infragistics.Win.DrawPhase.AfterDrawElement | Infragistics.Win.DrawPhase.BeforeDrawElement;
            }

            //The actual drawing code
            bool Infragistics.Win.IUIElementDrawFilter.DrawElement(Infragistics.Win.DrawPhase drawPhase, ref Infragistics.Win.UIElementDrawParams drawParams)
            {
                //If there's no DropHighlight node or no position
                //specified, then we don't need to draw anything special. 
                //Just exit Function
                if ((mvarDropHighLightNode == null) || (mvarDropLinePosition == DropLinePositionEnum.None))
                {
                    return false;
                }

                //Create a new QueryStateAllowedForNodeEventArgs object
                //to pass to the event
                QueryStateAllowedForNodeEventArgs eArgs = new QueryStateAllowedForNodeEventArgs();

                //Initialize the object with the correct info
                eArgs.Node = mvarDropHighLightNode;
                eArgs.DropLinePosition = this.mvarDropLinePosition;

                //Default to all states allowed. 
                eArgs.StatesAllowed = DropLinePositionEnum.All;

                //Raise the event
                this.QueryStateAllowedForNode(this, eArgs);

                //Check to see if the user allowed the current state
                //for this node. If not, exit function
                if ((eArgs.StatesAllowed & mvarDropLinePosition) != mvarDropLinePosition)
                {
                    return false;
                }

                Infragistics.Win.UIElement aUIElement;
                UltraTreeNode aNode;

                //Get the element being drawn
                aUIElement = drawParams.Element;

                //Determine which drawing phase we are in. 
                switch (drawPhase)
                {
                    case Infragistics.Win.DrawPhase.BeforeDrawElement:
                        {
                            //We are in BeforeDrawElement, so we are only concerned with 
                            //drawing the OnNode state. 
                            if ((mvarDropLinePosition & DropLinePositionEnum.OnNode) == DropLinePositionEnum.OnNode)
                            {
                                //Check to see if we are drawing a NodeTextUIElement
                                if (aUIElement.GetType() == typeof(Infragistics.Win.UltraWinTree.NodeTextUIElement))
                                {
                                    //Get a reference to the node that this
                                    //NodeTextUIElement is associated with
                                    aNode = (UltraTreeNode)aUIElement.GetContext(typeof(UltraTreeNode));

                                    //See if this is the DropHighlightNode
                                    if (aNode.Equals(mvarDropHighLightNode))
                                    {
                                        //Set the ForeColor and Backcolor of the node 
                                        //to the DropHighlight colors 
                                        //Note that AppearanceData only affects the
                                        //node for this one paint. It will not
                                        //change any properties of the node
                                        drawParams.AppearanceData.BackColor = mvarDropHighLightBackColor;
                                        drawParams.AppearanceData.ForeColor = mvarDropHighLightForeColor;
                                    }
                                }
                            }
                            break;
                        }
                    case Infragistics.Win.DrawPhase.AfterDrawElement:
                        {
                            //We're in AfterDrawElement, so the only states we are conderned with are
                            //Below and Above
                            //Check to see if we are drawing the Tree Element
                            if (aUIElement.GetType() == typeof(Infragistics.Win.UltraWinTree.UltraTreeUIElement))
                            {
                                //Get the NodeSelectableAreaUIElement for the 
                                //current DropNode. We will use this for
                                //positioning and sizing the DropLine
                                NodeSelectableAreaUIElement tElement;
                                tElement = (NodeSelectableAreaUIElement)drawParams.Element.GetDescendant(typeof(NodeSelectableAreaUIElement), mvarDropHighLightNode);
                                if (tElement != null)
                                {
                                    //The left edge of the DropLine
                                    int LeftEdge = tElement.Rect.Left - 4;

                                    //We need a reference to the control to 
                                    //determine the right edge of the line
                                    UltraTree aTree = (UltraTree)tElement.GetContext(typeof(UltraTree));
                                    int RightEdge = aTree.DisplayRectangle.Right - 4;

                                    //Get a reference to the Graphics object
                                    //we are drawing to. 
                                    System.Drawing.Graphics g = drawParams.Graphics;
                                    if ((mvarDropLinePosition & DropLinePositionEnum.AboveNode) == DropLinePositionEnum.AboveNode)
                                    {
                                        //Draw line above node
                                        DrawLine(mvarDropHighLightNode.Bounds.Top, LeftEdge, RightEdge, drawParams.Graphics);
                                    }
                                    if ((mvarDropLinePosition & DropLinePositionEnum.BelowNode) == DropLinePositionEnum.BelowNode)
                                    {
                                        //Draw Line below node
                                        DrawLine(mvarDropHighLightNode.Bounds.Bottom, LeftEdge, RightEdge, drawParams.Graphics);
                                    }
                                }
                            }
                            break;
                        }
                }
                return false;
            }

            /// <summary>
            /// Draw the line where the item will be dropped.
            /// </summary>
            /// <param name="LineVPosition">Vertical position</param>
            /// <param name="LeftEdge">Left edge</param>
            /// <param name="RightEdge">Right edge</param>
            /// <param name="g">Graphics object</param>
            private void DrawLine(int LineVPosition, int LeftEdge, int RightEdge, Graphics g)
            {
                // Profile: make pen a class variable, reuse it for each element
                using (System.Drawing.Pen pen = new System.Drawing.Pen(mvarDropLineColor, mvarDropLineWidth))
                {
                    g.DrawLine(pen, LeftEdge, LineVPosition, RightEdge, LineVPosition);
                    pen.Width = 1;
                    g.DrawLine(pen, LeftEdge, LineVPosition - 3, LeftEdge, LineVPosition + 2);
                    g.DrawLine(pen, LeftEdge + 1, LineVPosition - 2, LeftEdge + 1, LineVPosition + 1);
                    g.DrawLine(pen, RightEdge, LineVPosition - 3, RightEdge, LineVPosition + 2);
                    g.DrawLine(pen, RightEdge - 1, LineVPosition - 2, RightEdge - 1, LineVPosition + 1);
                }
            }
            #endregion
        }   // end class MyDrawFilter

        #region Event Declarations

        public event System.EventHandler<DragDropEventArgs> BeforeDragDrop;
        public event System.EventHandler<DragDropEventArgs> BeforeDragOver;
        public event System.EventHandler<BeforeSelectionDragStartEventArgs> BeforeSelectionDragStart;

        #endregion

        private readonly MyDrawFilter _myDrawFilter = new MyDrawFilter();

        public TFwkTree()
        {
            // DrawFilter initialisation
            base.DrawFilter = _myDrawFilter;
            _myDrawFilter.Invalidate += new EventHandler(_myDrawFilter_Invalidate);
            _myDrawFilter.QueryStateAllowedForNode += new MyDrawFilter.QueryStateAllowedForNodeEventHandler(_myDrawFilter_QueryStateAllowedForNode);

            // Additional (default) properties
            Override.SelectionType = SelectType.Extended;       //.ExtendedAutoDrag;
            //SAM://Override.SelectionType = SelectType.Single;
            AllowDrop = true;
            FullRowSelect = true;

            // Set up event handlers
            SelectionDragStart += new EventHandler(TFwkTree_SelectionDragStart);
            DragOver += new System.Windows.Forms.DragEventHandler(TFwkTree_DragOver);
            DragLeave += new EventHandler(TFwkTree_DragLeave);
            QueryContinueDrag += new System.Windows.Forms.QueryContinueDragEventHandler(TFwkTree_QueryContinueDrag);
            DragDrop += new DragEventHandler(TFwkTree_DragDrop);
        }

        /// <summary>
        /// Drop the selected item(s) on the dropnode, called by the framework.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">DragEventArgs</param>
        void TFwkTree_DragDrop(object sender, DragEventArgs e)
        {
            //The Node to Drop On
            UltraTreeNode DropNode = _myDrawFilter.DropHightLightNode;

            if (BeforeDragDrop != null)
            {
                DragDropEventArgs e2 = new DragDropEventArgs(e, DropNode, _myDrawFilter.DropLinePosition);

                //SAM- 05 NOV: Nodes are moving. Hence commented out.
                //BeforeDragDrop(this, e2);
                
                if (e2.Effect == DragDropEffects.None)
                {
                    _myDrawFilter.ClearDropHighlight();

                    return;     // Cancel drop operation - handled in event handler
                }
            }

            //Get the Data and put it into a SelectedNodes collection,
            //then clone it and work with the clone
            //These are the nodes that are being dragged and dropped
            SelectedNodesCollection SelectedNodes = (SelectedNodesCollection)e.Data.GetData(typeof(SelectedNodesCollection));
            SelectedNodes = SelectedNodes.Clone() as SelectedNodesCollection;

            //Sort the selected nodes into their visible position. 
            //This is done so that they stay in the same order when they are repositioned. 
            SelectedNodes.SortByPosition();

            //Determine where we are dropping based on the current DropLinePosition of the DrawFilter
            switch (_myDrawFilter.DropLinePosition)
            {
                case DropLinePositionEnum.OnNode: //Drop ON the node
                    {
                        //Loop through the SelectedNodes and reposition
                        //them to the node that was dropped on.
                        //Note that the DrawFilter keeps track of what
                        //node the mouse is over, so we can just use
                        //DropHighLightNode as the drop target. 
                        for (int i = 0; i <= (SelectedNodes.Count - 1); i++)
                        {
                            if ((e.Effect & DragDropEffects.Copy) == DragDropEffects.Copy)
                            {
                                DropNode.Nodes.Add(SelectedNodes[i].Clone());
                            }
                            else
                            {                                
                                SelectedNodes[i].Reposition(DropNode.Nodes);
                            }
                        }
                        break;
                    }
                case DropLinePositionEnum.BelowNode: //Drop Below the node
                    {
                        for (int i = 0; i <= (SelectedNodes.Count - 1); i++)
                        {
                            UltraTreeNode aNode = SelectedNodes[i];
                            aNode.Reposition(DropNode, Infragistics.Win.UltraWinTree.NodePosition.Next);
                            //Change the DropNode to the node that was
                            //just repositioned so that the next 
                            //added node goes below it. 
                            DropNode = aNode;
                        }
                        break;
                    }
                case DropLinePositionEnum.AboveNode: //New Index should be the same as the Drop
                    {
                        for (int i = 0; i <= (SelectedNodes.Count - 1); i++)
                        {
                            SelectedNodes[i].Reposition(DropNode, Infragistics.Win.UltraWinTree.NodePosition.Previous);
                        }
                        break;
                    }
            }

            _myDrawFilter.ClearDropHighlight();
        }

	    /// <summary>
        /// Test to see if we want to continue dragging
	    /// </summary
	    /// <param name="sender">Disregarded</param>
        /// <param name="e">QueryContinueDragEventArgs</param>
        void TFwkTree_QueryContinueDrag(object sender, System.Windows.Forms.QueryContinueDragEventArgs e)
        {
            //Did the user press escape? 
            if (e.EscapePressed)
            {
                //User pressed escape
                //Cancel the Drag
                e.Action = DragAction.Cancel;
                //Clear the Drop highlight, since we are no longer
                //dragging
                _myDrawFilter.ClearDropHighlight();
            }
        }

        //Fires when the user drags outside the control. 
        void TFwkTree_DragLeave(object sender, EventArgs e)
        {
            //When the mouse goes outside the control, clear the 
            //drophighlight. 
            //Since the DropHighlight is cleared when the 
            //mouse is not over a node, anyway, 
            //this is probably not needed
            //But, just in case the user goes from a node directly
            //off the control...
            _myDrawFilter.ClearDropHighlight();
        }

        /// <summary>
        /// Called when an object is dragged over our tree.
        /// TODO: implement this as a call into an event.
        /// </summary>
        /// <param name="sender">object, the UltraTree sending this event.</param>
        /// <param name="e">System.Windows.Forms.DragEventArgs</param>
        void TFwkTree_DragOver(object sender, System.Windows.Forms.DragEventArgs e)
        {
            System.Drawing.Point PointInTree = PointToClient(new Point(e.X, e.Y));
            UltraTreeNode aNode = GetNodeFromPoint(PointInTree);
            e.Effect = DragDropEffects.None;        // Some default value

            //Make sure the mouse is over a node
            if (aNode == null)
            {
                //The Mouse is not over a node
                //Erase any DropHighlight
                _myDrawFilter.ClearDropHighlight();
                return;
            }

            if (BeforeDragOver != null)
            {
                DragDropEventArgs e2 = new DragDropEventArgs(e, aNode, _myDrawFilter.DropLinePosition);
                BeforeDragOver(this, e2);
                e.Effect = e2.Effect;
                if (e2.Effect == DragDropEffects.None)
                {
                    //Erase any DropHighlight
                    _myDrawFilter.ClearDropHighlight();
                    return;
                }
            }


            // Check for control key
            if ((e.KeyState & 8) == 8 && (e.Effect & DragDropEffects.Copy) == DragDropEffects.Copy)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.Move;
            }
            _myDrawFilter.SetDropHighlightNode(aNode, PointInTree);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender">object, the UltraTree sending this event.</param>
        /// <param name="e">EventArgs</param>
        void TFwkTree_SelectionDragStart(object sender, EventArgs e)
        {
            DragDropEffects dde = DragDropEffects.All;     // Default value;
            if (BeforeSelectionDragStart != null)
            {
                BeforeSelectionDragStartEventArgs e2 = new BeforeSelectionDragStartEventArgs(SelectedNodes);
                BeforeSelectionDragStart(this, e2);
                dde = e2.AllowedEffects;
            }

            if (dde != DragDropEffects.None)
            {
                Debug.WriteLine(string.Format("dde = {0}", dde), "TFwkTree_SelectionDragStart");
                DoDragDrop(SelectedNodes, dde);
            }
        }

        //This event is fired by the DrawFilter to let us determine
        //what kinds of drops we want to allow on any particular node

        /// <summary>
        /// This event is fired by the DrawFilter to let us determine
        /// what kinds of drops we want to allow on any particular node
        /// TODO: Call an event (?)
        /// </summary>
        /// <param name="sender">object, the UltraTree sending this event.</param>
        /// <param name="e">TFwkTree.MyDrawFilter.QueryStateAllowedForNodeEventArgs</param>
        void _myDrawFilter_QueryStateAllowedForNode(object sender, TFwkTree.MyDrawFilter.QueryStateAllowedForNodeEventArgs e)
        {
            e.StatesAllowed = DropLinePositionEnum.All;
            //e.StatesAllowed = DropLinePositionEnum.OnNode;
        }

        /// <summary>
        /// Occassionally, the DrawFilter will let us know that the control needs to be invalidated.
        /// </summary>
        /// <param name="sender">object, the UltraTree sending this event.</param>
        /// <param name="e">EventArgs</param>
        void _myDrawFilter_Invalidate(object sender, EventArgs e)
        {
            //Any time the drophighlight changes, the control needs 
            //to know that it has to repaint. 
            //It would be more efficient to only invalidate the area
            //that needs it, but this works and is very clean.
            Invalidate();
        }

        public List<UltraTreeNode> GetNodes(bool bIncludeSubnodes)
        {
            return GetNodes(bIncludeSubnodes, Nodes);
        }

        public List<UltraTreeNode> GetNodes(bool bIncludeSubnodes, TreeNodesCollection NodesColl)
        {
            List<UltraTreeNode> nodes = new List<UltraTreeNode>(this.GetNodeCount(true));
            GetNodes(nodes, bIncludeSubnodes, NodesColl);

            return nodes;
        }

        private void GetNodes(List<UltraTreeNode> nodes, bool bIncludeSubnodes, TreeNodesCollection NodesColl)
        {
            foreach (UltraTreeNode node in NodesColl)
            {
                nodes.Add(node);
                if (bIncludeSubnodes)
                {
                    GetNodes(nodes, true, node.Nodes);
                }
            }
        }

        private void InitializeComponent()
        {
            ((System.ComponentModel.ISupportInitialize)(this)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this)).EndInit();

        }


    }

    //Enumerates the possible Drop States
    [System.Flags]
    public enum DropLinePositionEnum
    {
        None = 0,
        OnNode = 1,
        AboveNode = 2,
        BelowNode = 4,
        All = OnNode | AboveNode | BelowNode
    }


    /// <summary>
    /// Used in the BeforeDragDrop and BeforeDragOver events.
    /// </summary>
    public class DragDropEventArgs : DragEventArgs
    {
        public UltraTreeNode DropNode;
        public DropLinePositionEnum LinePosition;

        public DragDropEventArgs(DragEventArgs e, UltraTreeNode node, DropLinePositionEnum pos)
            : base(e.Data, e.KeyState, e.X, e.Y, e.AllowedEffect, e.Effect)
        {
            DropNode = node;
            LinePosition = pos;
        }
    }

    /// <summary>
    /// Used in the BeforeSelectionDragStart event.
    /// </summary>
    public class BeforeSelectionDragStartEventArgs : EventArgs
    {
        public SelectedNodesCollection SelectedNodes;
        public DragDropEffects AllowedEffects = DragDropEffects.Move;

        public BeforeSelectionDragStartEventArgs(SelectedNodesCollection Nodes)
            : base()
        {
            SelectedNodes = Nodes;
        }
    }


}
